package pilas.modelo;

/**
 * Representa la pila como tal con sus operaciones principales de apilar y
 * retirar
 * 
 * @author Libardo
 * 
 * @param <T>
 *            tipo de objeto a almacenar en la pila
 */
public class Pila<T> {
	/**
	 * Atributo cabeza, que apunta al tope la pila
	 */
	private Nodo<T> cabeza;
	/**
	 * Almacena el total de elemento de la pila
	 */
	private int tamanio;

	/**
	 * Constructor por defecto
	 */
	public Pila() {
		cabeza = null;
		tamanio = 0;
	}

	/**
	 * Devuelve el total de elementos de la pila
	 * 
	 * @return total de elementos de la pila
	 */
	public int getTamanio() {
		return tamanio;
	}

	/**
	 * Verifica si la pila esta vacia
	 * 
	 * @return true cuando es vacia, false en caso contario
	 */
	public boolean esVacia() {
		if (cabeza == null)
			return true;
		else
			return false;
	}

	//
	/**
	 * Apila un elemento nuevo
	 * 
	 * @param valor
	 *            valora a apilar
	 */
	public void apilar(T valor) {
		Nodo<T> nuevo = new Nodo<T>();
		nuevo.setValor(valor);
		if (esVacia()) {
			cabeza = nuevo;
		} else {
			nuevo.setSiguiente(cabeza);
			cabeza = nuevo;
		}
		tamanio++;
	}

	/**
	 * Elimina un elemento de la pila
	 */
	public void retirar() {
		if (!esVacia()) {
			cabeza = cabeza.getSiguiente();
			tamanio--;
		}
	}

	/**
	 * Devuelve el elemento almacenado en el tope de la pila
	 * 
	 * @return tope de la pila
	 */
	public T cima() {
		if (!esVacia())
			return cabeza.getValor();
		else
			return null;
	}
}