package pilas.test;

import java.util.Stack;

/**
 * Esta clase convierte una expresión aritmética de notación infija a postfija
 * Utiliza la clase Stack de java, pero se puede usar las clases Pila y Nodo que
 * implementamos en este capítulo.
 * 
 * @author Libardo
 * 
 */
public class InfijaPostfija {

	public static void main(String[] args) {
		
		String cad = "(A+B)*(C-D)^E";
		System.out.println(cad + " a postfija:" + aPostfija(cad));
		cad = "X^Y+Z-A*D-R";
		System.out.println(cad + " a postfija:" + aPostfija(cad));		
	}

	/**
	 * Convierte de infija a postfija
	 * 
	 * @param infija
	 *            expresión en notación infija
	 * @return expresión en notación postfija
	 */
	public static String aPostfija(String infija) {
		Stack<Character> pila = new Stack<Character>();
		char carActual;
		String postfija = "";
		for (int i = 0; i < infija.length(); i++) {
			carActual = infija.charAt(i);

			if (carActual == '(')
				pila.push(carActual);
			else if (carActual == ')') {
				while (pila.peek() != '(')
					postfija += pila.pop();
				pila.pop();
			} else if (esUnOperador(carActual)) {
				while (!pila.empty() && esUnOperador(pila.peek())
						&& !esMayorPrioridad(carActual, pila.peek())) {
					postfija += pila.pop();
				}
				pila.push(carActual);
			}

			else
				postfija += carActual;
		}
		while (!pila.empty())
			postfija += pila.pop();
		return postfija;
	}

	public static boolean esUnOperador(char c) {
		if (c == '+' || c == '-' || c == '*' || c == '/' || c == '^')
			return true;
		else
			return false;

	}

	public static boolean esMayorPrioridad(char c1, char c2) {
		int prioridad = 0;
		int prioridad2 = 0;
		if (c1 == '^')
			prioridad = 3;
		if (c1 == '*' || c1 == '/')
			prioridad = 2;
		if (c1 == '+' || c1 == '-')
			prioridad = 1;

		if (c2 == '^')
			prioridad2 = 3;
		if (c2 == '*' || c2 == '/')
			prioridad2 = 2;
		if (c2 == '+' || c2 == '-')
			prioridad2 = 1;
		return (prioridad > prioridad2);
	}

}
